/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvMath.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_tX1R58cL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_miKZ2339_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qphkHpm2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DTaAlnq3_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_kYmsiV26_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_g7clO2F5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_W0I1iCBZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_weAXRb01_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PX91WR7U_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_lo0rFvHC_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Is58nnmr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_xAuqBhSU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Epszo0QF_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_msRrXiQL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_VKQj7ruz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_TUf47Sct_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_n08bnljK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_w1nfmCeE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_imUjjVHM_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_sc5eu1ri_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8Rx1TuNp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oK742dJU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_kB3lilGd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_vAJTzNNG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_cxNIXl50_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_bE9RcUWH_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_2ZfGsFrn;
  SignalLine sLine_5waqroOg;
  SignalLine sLine_kt9oXqzG;
  ControlBinop cBinop_tX1R58cL;
  ControlBinop cBinop_miKZ2339;
  ControlBinop cBinop_qphkHpm2;
  ControlBinop cBinop_DTaAlnq3;
  ControlBinop cBinop_W0I1iCBZ;
  ControlBinop cBinop_PX91WR7U;
  ControlBinop cBinop_lo0rFvHC;
  ControlBinop cBinop_Is58nnmr;
  ControlVar cVar_xAuqBhSU;
  ControlVar cVar_Epszo0QF;
  SignalVarf sVarf_Lk7zhmzY;
  SignalVarf sVarf_3sonaP2t;
  SignalVarf sVarf_sP4a2GeT;
  SignalVarf sVarf_IBoqzxCt;
  ControlBinop cBinop_w1nfmCeE;
  ControlBinop cBinop_oK742dJU;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
